@@ -7,31 +7,29 @@
 /*	#include "bounce_service.h"
 /*
 /*	int     bounce_notify_service(flags, queue_name, queue_id, encoding,
-/*					sender)
+/*					sender, dsn_envid, dsn_ret)
 /*	int	flags;
 /*	char	*queue_name;
 /*	char	*queue_id;
 /*	char	*encoding;
 /*	char	*sender;
-/*	int	flush;
+/*	char	*dsn_envid;
+/*	int	dsn_ret;
 /* DESCRIPTION
-/*	This module implements the server side of the bounce_notify()
-/*	(send bounce message) request. The logfile is removed after a
-/*	warning is posted.
+/*	This module implements the server side of the bounce_flush()
+/*	(send bounce message) request.
 /*
 /*	When a message bounces, a full copy is sent to the originator,
-/*	and an optional  copy of the diagnostics with message headers is
+/*	and an optional copy of the diagnostics with message headers is
 /*	sent to the postmaster.  The result is non-zero when the operation
-/*	should be tried again.
+/*	should be tried again. Otherwise, the logfile is removed.
 /*
 /*	When a bounce is sent, the sender address is the empty
 /*	address.  When a bounce bounces, an optional double bounce
 /*	with the entire undeliverable mail is sent to the postmaster,
 /*	with as sender address the double bounce address.
 /* DIAGNOSTICS
-/*	Fatal error: error opening existing file. Warnings: corrupt
-/*	message file. A corrupt message is saved to the "corrupt"
-/*	queue for further inspection.
+/*	Fatal error: error opening existing file.
 /* BUGS
 /* SEE ALSO
 /*	bounce(3) basic bounce service client interface
@@ -72,6 +70,7 @@
 #include <mail_addr.h>
 #include <mail_error.h>
 #include <bounce.h>
+#include <dsn_mask.h>
 
 /* Application-specific. */
 
@@ -83,7 +82,8 @@
 
 int     bounce_notify_service(int flags, char *service, char *queue_name,
 			              char *queue_id, char *encoding,
-			              char *recipient)
+			              char *recipient, char *dsn_envid,
+			              int dsn_ret)
 {
     BOUNCE_INFO *bounce_info;
     int     bounce_status = 1;
@@ -92,17 +92,45 @@ int     bounce_notify_service(int flags, char *service, char *queue_name,
     int     notify_mask = name_mask(VAR_NOTIFY_CLASSES, mail_error_masks,
 				    var_notify_classes);
     char   *postmaster;
+    int     count;
 
     /*
      * Initialize. Open queue file, bounce log, etc.
+     * 
+     * XXX DSN The bounce service produces RFC 3464-style "failed mail" reports
+     * from information in two following types of logfile:
+     * 
+     * 1 - bounce: this file is used for RFC 3464-style reports of permanent
+     * delivery errors by the bounce(8) service. This reports to the sender
+     * all recipients that have no DSN NOTIFY information (compatibility) and
+     * all recipients that have DSN NOTIFY=FAILURE; this reports to
+     * postmaster all recipients, if postmaster notification is enabled.
+     * 
+     * 2 - defer: this file is used for three types of report:
+     * 
+     * 2a) RFC 3464-style "mail is too old" reports by the bounce(8) service.
+     * This reports to the sender all recipients that have no DSN NOTIFY
+     * information (compatibility) and all recipients that have DSN
+     * NOTIFY=FAILURE; this reports to postmaster all recipients, if
+     * postmaster notification is enabled.
+     * 
+     * Other reports that other servers produce from the defer logfile:
+     * 
+     * 2b) On-demand reports of all delayed deliveries by the showq(8) service
+     * and mailq(1) command. This reports all recipients that have a
+     * transient delivery error.
+     * 
+     * 2c) RFC 3464-style "delayed mail" notifications by the defer(8) service.
+     * This reports to the sender all recipients that have no DSN NOTIFY
+     * information (compatibility) and all recipients that have DSN
+     * NOTIFY=DELAY; this reports to postmaster all recipients, if postmaster
+     * notification is enabled.
      */
     bounce_info = bounce_mail_init(service, queue_name, queue_id,
-				   encoding, BOUNCE_MSG_FAIL);
+				   encoding, dsn_envid, BOUNCE_REPORT_FAIL);
 
 #define NULL_SENDER		MAIL_ADDR_EMPTY	/* special address */
 #define NULL_TRACE_FLAGS	0
-#define BOUNCE_HEADERS		1
-#define BOUNCE_ALL		0
 
     /*
      * The choice of sender address depends on the recipient address. For a
@@ -131,13 +159,14 @@ int     bounce_notify_service(int flags, char *service, char *queue_name,
     }
 
     /*
-     * Single bounce failed. Optionally send a double bounce to postmaster.
+     * Single bounce failed. Optionally send a double bounce to postmaster,
+     * subject to notify_classes restrictions.
      */
 #define ANY_BOUNCE (MAIL_ERROR_2BOUNCE | MAIL_ERROR_BOUNCE)
-#define SKIP_IF_BOUNCE ((notify_mask & ANY_BOUNCE) == 0)
+#define SEND_POSTMASTER_ANY_BOUNCE_NOTICE (notify_mask & ANY_BOUNCE)
 
     else if (*recipient == 0) {
-	if (SKIP_IF_BOUNCE) {
+	if (!SEND_POSTMASTER_ANY_BOUNCE_NOTICE) {
 	    bounce_status = 0;
 	} else {
 	    postmaster = var_2bounce_rcpt;
@@ -152,18 +181,28 @@ int     bounce_notify_service(int flags, char *service, char *queue_name,
 		 * reason for the bounce, and the headers of the original
 		 * message. Don't bother sending the boiler-plate text.
 		 */
-		if (!bounce_header(bounce, bounce_info, postmaster)
-		    && bounce_diagnostic_log(bounce, bounce_info) == 0
+		count = -1;
+		if (bounce_header(bounce, bounce_info, postmaster) == 0
+		    && (count = bounce_diagnostic_log(bounce, bounce_info,
+						   DSN_NOTIFY_OVERRIDE)) > 0
 		    && bounce_header_dsn(bounce, bounce_info) == 0
-		    && bounce_diagnostic_dsn(bounce, bounce_info) == 0)
-		    bounce_original(bounce, bounce_info, BOUNCE_ALL);
-		bounce_status = post_mail_fclose(bounce);
+		    && bounce_diagnostic_dsn(bounce, bounce_info,
+					     DSN_NOTIFY_OVERRIDE) > 0) {
+		    bounce_original(bounce, bounce_info, DSN_RET_FULL);
+		    bounce_status = post_mail_fclose(bounce);
+		} else {
+		    /* No applicable recipients found - cancel this notice. */
+		    (void) vstream_fclose(bounce);
+		    if (count == 0)
+			bounce_status = 0;
+		}
 	    }
 	}
     }
 
     /*
-     * Non-bounce failed. Send a single bounce.
+     * Non-bounce failed. Send a single bounce to the sender, subject to DSN
+     * NOTIFY restrictions.
      */
     else {
 	if ((bounce = post_mail_fopen_nowait(NULL_SENDER, recipient,
@@ -175,24 +214,35 @@ int     bounce_notify_service(int flags, char *service, char *queue_name,
 	     * pretends that we are a polite mail system, the text with
 	     * reason for the bounce, and a copy of the original message.
 	     */
+	    count = -1;
 	    if (bounce_header(bounce, bounce_info, recipient) == 0
 		&& bounce_boilerplate(bounce, bounce_info) == 0
-		&& bounce_diagnostic_log(bounce, bounce_info) == 0
+		&& (count = bounce_diagnostic_log(bounce, bounce_info,
+						  DSN_NOTIFY_FAILURE)) > 0
 		&& bounce_header_dsn(bounce, bounce_info) == 0
-		&& bounce_diagnostic_dsn(bounce, bounce_info) == 0)
-		bounce_original(bounce, bounce_info, BOUNCE_ALL);
-	    bounce_status = post_mail_fclose(bounce);
+		&& bounce_diagnostic_dsn(bounce, bounce_info,
+					 DSN_NOTIFY_FAILURE) > 0) {
+		bounce_original(bounce, bounce_info, dsn_ret ?
+				dsn_ret : DSN_RET_FULL);
+		bounce_status = post_mail_fclose(bounce);
+	    } else {
+		/* No applicable recipients found - cancel this notice. */
+		(void) vstream_fclose(bounce);
+		if (count == 0)
+		    bounce_status = 0;
+	    }
 	}
 
 	/*
-	 * Optionally, send a postmaster notice.
+	 * Optionally, send a postmaster notice, subject to notify_classes
+	 * restrictions.
 	 * 
 	 * This postmaster notice is not critical, so if it fails don't
 	 * retransmit the bounce that we just generated, just log a warning.
 	 */
-#define WANT_IF_BOUNCE ((notify_mask & MAIL_ERROR_BOUNCE))
+#define SEND_POSTMASTER_SINGLE_BOUNCE_NOTICE (notify_mask & MAIL_ERROR_BOUNCE)
 
-	if (bounce_status == 0 && (WANT_IF_BOUNCE)
+	if (bounce_status == 0 && SEND_POSTMASTER_SINGLE_BOUNCE_NOTICE
 	    && strcasecmp(recipient, mail_addr_double_bounce()) != 0) {
 
 	    /*
@@ -207,12 +257,21 @@ int     bounce_notify_service(int flags, char *service, char *queue_name,
 						 postmaster,
 						 CLEANUP_FLAG_MASK_INTERNAL,
 						 NULL_TRACE_FLAGS)) != 0) {
+		count = -1;
 		if (bounce_header(bounce, bounce_info, postmaster) == 0
-		    && bounce_diagnostic_log(bounce, bounce_info) == 0
+		    && (count = bounce_diagnostic_log(bounce, bounce_info,
+						   DSN_NOTIFY_OVERRIDE)) > 0
 		    && bounce_header_dsn(bounce, bounce_info) == 0
-		    && bounce_diagnostic_dsn(bounce, bounce_info) == 0)
-		    bounce_original(bounce, bounce_info, BOUNCE_HEADERS);
-		postmaster_status = post_mail_fclose(bounce);
+		    && bounce_diagnostic_dsn(bounce, bounce_info,
+					     DSN_NOTIFY_OVERRIDE) > 0) {
+		    bounce_original(bounce, bounce_info, DSN_RET_HDRS);
+		    postmaster_status = post_mail_fclose(bounce);
+		} else {
+		    /* No applicable recipients found - cancel this notice. */
+		    (void) vstream_fclose(bounce);
+		    if (count == 0)
+			postmaster_status = 0;
+		}
 	    }
 	    if (postmaster_status)
 		msg_warn("postmaster notice failed while bouncing to %s",
